package com.atguigu.tingshu.album.api;

import com.atguigu.tingshu.album.service.AlbumInfoService;
import com.atguigu.tingshu.common.login.GuiGuLogin;
import com.atguigu.tingshu.common.result.Result;
import com.atguigu.tingshu.common.util.AuthContextHolder;
import com.atguigu.tingshu.model.album.AlbumInfo;
import com.atguigu.tingshu.query.album.AlbumInfoQuery;
import com.atguigu.tingshu.vo.album.AlbumInfoVo;
import com.atguigu.tingshu.vo.album.AlbumListVo;
import com.atguigu.tingshu.vo.album.AlbumStatVo;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

//@GuiGuLogin
@Tag(name = "专辑管理")
@RestController  //SpringMVC 默认JSON框架 Jackson
@RequestMapping("api/album")
@SuppressWarnings({"all"})
public class AlbumInfoApiController {

    @Autowired
    private AlbumInfoService albumInfoService;


    /**
     * 保存专辑
     * 该接口必须登录才能访问，调用接口内部获取到当前登录系统用户ID
     *
     * @param albumInfoVo
     * @return
     */
    @GuiGuLogin
    @Operation(summary = "专辑保存")
    @PostMapping("/albumInfo/saveAlbumInfo")
    public Result saveAlbumInfo(@RequestBody @Validated AlbumInfoVo albumInfoVo) {
        //1.TODO 从ThreadLocal中获取固定用户ID
        Long userId = AuthContextHolder.getUserId();
        //2.调用业务层完成保存专辑
        albumInfoService.saveAlbumInfo(albumInfoVo, userId);
        return Result.ok();
    }


    /**
     *  该接口必须登陆访问
     * 分页条件查询当前登录用户专辑列表
     *
     * @param page
     * @param limit
     * @param albumInfoQuery
     * @return
     */
    @GuiGuLogin(required=true)
    @Operation(summary = "分页条件查询当前登录用户专辑列表")
    @PostMapping("/albumInfo/findUserAlbumPage/{page}/{limit}")
    public Result<Page<AlbumListVo>> findUserAlbumPage(
            @PathVariable Long page,
            @PathVariable Long limit,
            @RequestBody AlbumInfoQuery albumInfoQuery
    ) {
        //1.获取当前登录用户ID
        Long userId = AuthContextHolder.getUserId();
        albumInfoQuery.setUserId(userId);
        //2.创建MP分页需要分页对象Page,在控制层封装当前页，页大小
        Page<AlbumListVo> pageInfo = new Page<>(page, limit);
        //3.调用业务层分页方法 封装分页对象中：总记录数、当前页数据、总页数等
        pageInfo = albumInfoService.findUserAlbumPage(pageInfo, albumInfoQuery);
        return Result.ok(pageInfo);
    }


    /**
     * 根据专辑ID删除专辑相关信息
     *
     * @param id
     * @return
     */
    @Operation(summary = "根据专辑ID删除专辑相关信息")
    @DeleteMapping("/albumInfo/removeAlbumInfo/{id}")
    public Result removeAlbumInfo(@PathVariable Long id) {
        //1.调用业务层删除专辑
        albumInfoService.removeAlbumInfo(id);
        return Result.ok();
    }


    /**
     * 根据专辑ID查询专辑信息，包含标签列表
     *
     * @param id
     * @return
     */
    @Operation(summary = "根据专辑ID查询专辑信息，包含标签列表")
    @GetMapping("/albumInfo/getAlbumInfo/{id}")
    public Result<AlbumInfo> getAlbumInfo(@PathVariable Long id) {
        AlbumInfo albumInfo = albumInfoService.getAlbumInfo(id);
        return Result.ok(albumInfo);
    }

    /**
     * 更新专辑信息
     * @param id
     * @param albumInfoVo
     * @return
     */
    @Operation(summary = "更新专辑信息")
    @PutMapping("/albumInfo/updateAlbumInfo/{id}")
    public Result updateAlbumInfo(@PathVariable Long id, @RequestBody @Validated AlbumInfoVo albumInfoVo) {
        albumInfoService.updateAlbumInfo(id, albumInfoVo);
        return Result.ok();
    }

    /**
     * 该登录才能访问
     * 查询当前登录用户发布专辑列表
     * @return
     */
    @GuiGuLogin
    @Operation(summary = "查询当前登录用户保存过审专辑列表")
    @GetMapping("/albumInfo/findUserAllAlbumList")
    public Result<List<AlbumInfo>> findUserAllAlbumList() {
        Long userId = AuthContextHolder.getUserId();
        List<AlbumInfo> list = albumInfoService.findUserAllAlbumList(userId);
        return Result.ok(list);
    }


    /**
     * 根据专辑ID查询专辑统计信息
     * @param albumId
     * @return
     */
    @Operation(summary = "根据专辑ID查询专辑统计信息")
    @GetMapping("/albumInfo/getAlbumStatVo/{albumId}")
    public Result<AlbumStatVo> getAlbumStatVo(@PathVariable("albumId") Long albumId) {
        AlbumStatVo albumStatVo = albumInfoService.getAlbumStatVo(albumId);
        return Result.ok(albumStatVo);
    }
}

